import { randomUUID } from "crypto";
import { Vertex } from "./vertex";

export class Edge {

    private id: string;
    private category: string;
    private directional: boolean;
    private parentVertex: Vertex;
    private childVertex: Vertex;

    constructor(id: null | string, parent: Vertex, child: Vertex, category = "generic", directional = false) {
        this.id ? id : randomUUID();
        this.parentVertex = parent;
        this.childVertex = child;
        this.category = category;
        this.directional = directional;
    }

    public getId() { return this.id; }
    public getCategory() { return this.category; }
    public getDirectional() { return this.directional; }
    public getParentVertex() { return this.parentVertex; }
    public getChildVertex() { return this.childVertex; }

    public getAdjacent(vertex: Vertex) {
        if (vertex == this.childVertex) return this.parentVertex;
        else if (vertex == this.parentVertex) return this.childVertex;
        else throw new Error(`Edge ${this.id} is not connected to vertex ${vertex.getId()}`);
    }

}