import { Edge } from "./edge";
import { Vertex } from "./vertex";

export class Graph {

    private vertices: Map<string, Vertex>;
    private edges: Map<string, Edge>;

    constructor(vertices: Vertex[] = [], edges: Edge[] = []) {
        this.vertices = new Map();
        this.edges = new Map();
        vertices.forEach(this.addVertex);
        edges.forEach(this.addEdge);
    }

    public getAllVertices() {
        return Array.from(this.vertices.values());
    }

    public getVertex(id: string) {
        return this.vertices.get(id);
    }

    public addVertex(vertex: Vertex) {
        this.vertices.set(vertex.getId(), vertex);
    }

    public deleteVertex(vertex: string | Vertex) {
        if (vertex instanceof Vertex) vertex = vertex.getId();
        this.vertices.delete(vertex);
    }

    public getAllEdges() {
        return Array.from(this.edges.values());
    }

    public getEdge(id: string) {
        return this.edges.get(id);
    }

    public addEdge(edge: Edge) {
        edge.getParentVertex().addEdge(edge);
        edge.getChildVertex().addEdge(edge);
        this.edges.set(edge.getId(), edge);
    }

    public deleteEdge(edge: string | Edge) {
        if (typeof edge == "string") edge = this.edges.get(edge);
        if (!edge) throw new Error(`Failed to identify edge with an ID of ${edge.getId()}.`);
        edge.getParentVertex().removeEdge(edge);
        edge.getChildVertex().removeEdge(edge);
        this.edges.delete(edge.getId());
    }

}