import { randomUUID } from "crypto";
import { Edge } from "./edge";

export class Vertex {

    private id: string
    private degree: number;
    private edges: Edge[];
    private data: any; 

    constructor(id: null | string, data: any, edges: Edge[] = []) {
        this.id = id ? id : randomUUID();
        this.data = data;
        this.degree = edges.length;
        this.edges = edges;
    }

    public getId() { return this.id; }
    public getDegree() { return this.degree; }
    public getEdges() { return this.edges; }
    public getData() { return this.data; }

    public updateData(data: any) {
        this.data = data;
    } 

    public addEdge(edge: Edge) {
        this.edges.push(edge);
        this.degree++;
    }

    public removeEdge(edge: Edge) {
        const index = this.edges.indexOf(edge);
        if (index == -1) throw new Error(`Unable to identify edge ${edge.getId()} from vertex ${this.id}.`);
        this.edges.splice(index, 1);
        this.degree--;
    }

}